 
#include "Arduino.h"
#define NECHANT 128 // nombre d'échantillons de la table du signal
#define SHIFT_ACCUM 25
#define SET_FREQ 100
#define SET_AMP 101
#define START_SYNCHRO 102
#define STOP_SYNCHRO 103
#define PERIOD 100 // période du signal MLI en microsecondes (10 kHz)
#define NSYNCHRO 25 // période du signal de synchro = 2*NSYNCHRO*PERIOD

uint32_t icr;
uint32_t table_onde_A[NECHANT];
uint32_t table_onde_B[NECHANT];
uint32_t accumA,accumB,increm;
uint16_t diviseur[6] = {0,1,8,64,256,1024};
volatile uint8_t flag;
uint8_t strob = 0;
uint32_t indexA,indexB;
uint32_t compteur;
uint8_t start_synchro, synchro;

    			 
void init_pwm_timer1(uint32_t period) {
    TCCR1A = 0;
    TCCR1A |= (1 << COM1A1); //Clear OCnA/OCnB/OCnC on compare match, set OCnA/OCnB/OCnC at BOTTOM (non-inverting mode)
    TCCR1A |= (1 << COM1B1);
#if defined(__AVR_ATmega2560__) || defined(__AVR_ATmega32U4__)
    TCCR1A |= (1 << COM1C1);
#endif
    TCCR1B = 1 << WGM13; // phase and frequency correct pwm mode, top = ICR1
    int d = 1;
    icr = (F_CPU/1000000*period/2);
    while ((icr>0xFFFF)&&(d<6)) { // choix du diviseur d'horloge
        d++;
        icr = (F_CPU/1000000*period/2/diviseur[d]);
   } 
   ICR1 = icr; // valeur maximale du compteur
   TIMSK1 = 1 << TOIE1; // overflow interrupt enable
   sei(); // activation des interruptions
   TCNT1 = 0; // mise à zéro du compteur
   TCCR1B |= d; // déclenchement du compteur
}
    			 
ISR(TIMER1_OVF_vect) { // Timer 1 Overflow interrupt
  accumA += increm;
  accumB += increm;
  indexA = accumA >> SHIFT_ACCUM;
  OCR1A = table_onde_A[indexA];
  indexB = accumB >> SHIFT_ACCUM;
  OCR1B = table_onde_B[indexB];
  if ((indexA==0)&&(start_synchro==1)) {
    compteur = 0; // première image au début du cycle (champ A nul et croissant)
    flag = 0;
    start_synchro = 0;
    synchro = 1;
  }
  if (synchro) {
    if (compteur==0) {
      
      if (flag) {
    #if defined(__AVR_ATmega2560__)
        PORTH &= ~(1 << PORTH4); 
    #else
        PORTD &= ~(1 << PORTD7); 
    #endif   
        flag = 0;
      }
      else {
   #if defined(__AVR_ATmega2560__)
        PORTH |= (1 << PORTH4);  
   #else
        PORTD |= (1 << PORTD7);      
   #endif
        flag = 1;
      }
    }
    compteur += 1;
    if (compteur==NSYNCHRO) compteur=0;
  }
}
    			   
void set_sinus_table_A(float amp, float offset) {
  int i;
  float dt = 2*PI/NECHANT;
  for(i=0; i<NECHANT; i++) {
    table_onde_A[i] = icr*0.5*(1.0+offset+amp*sin(i*dt));
  }  
}    			  
    			   
void set_sinus_table_B(float amp, float offset) {
  int i;
  float dt = 2*PI/NECHANT;
  for(i=0; i<NECHANT; i++) {
    table_onde_B[i] = icr*0.5*(1.0+offset+amp*sin(i*dt));
  }  
}    			  
    			  
void setup() {
    char c;
    Serial.begin(115200);
    Serial.setTimeout(0);
    c = 0;
    Serial.write(c);
    c = 255;
    Serial.write(c);
    c = 0;
    Serial.write(c);
    pinMode(8,OUTPUT); 
    digitalWrite(8,HIGH); // ENABLE A
    pinMode(6,OUTPUT);
    digitalWrite(6,HIGH); // ENABLE B
#if defined(__AVR_ATmega2560__) // arduino MEGA
    pinMode(11,OUTPUT); // INA
    pinMode(12,OUTPUT); // INB
#else // arduino UNO et YUN
    pinMode(9,OUTPUT); // INA
    pinMode(10,OUTPUT); // INB
#endif
	pinMode(7,OUTPUT); // sortie synchro pour la caméra (port PD7 sur UNO, PH4 sur MEGA)
    digitalWrite(7,LOW);
    
    float frequence = 5; // en Hz
    accumA = 0;
    accumB = ((uint32_t)(NECHANT * 0.25)) << SHIFT_ACCUM;
    increm = (uint32_t) (((float)(0xFFFFFFFF))*((float)(frequence)*1e-6*(float)(PERIOD))); // incrément de l'accumulateur de phase
    flag=0;
    compteur = 0;
    start_synchro = 0;
    synchro = 0;
    init_pwm_timer1(PERIOD);
    set_sinus_table_A(0.2,0.5);
}
    			    
  void loop() {
  char com;
  uint32_t c1,c2,c3,c4;
  uint32_t freq; // fréquence en mHz
  float frequence; // fréquence en Hz
  uint16_t A; // amplitude
  uint16_t Off; // offset
  float amp,offset;
    if (Serial.available()>0) {
       
          com = Serial.read();
          if (com==SET_FREQ) {
              while (Serial.available()<4) {};
              c1 = Serial.read();
              c2 = Serial.read();
              c3 = Serial.read();
              c4 = Serial.read();
              freq = ((c1 << 24) | (c2 << 16) | (c3 << 8) | c4);
              frequence = (float)(freq);
              frequence /= 1000.0;
              increm = (uint32_t) (((float)(0xFFFFFFFF))*((float)(frequence)*1e-6*(float)(PERIOD))); // incrément de l'accumulateur de phase
          }
          else if (com==SET_AMP) {
              while (Serial.available()<2) {};
              c1 = Serial.read(); 
              c2 = Serial.read();
              A = (c1 << 8) | c2;
              amp = (float) A;
              while (Serial.available()<2) {};
              c1 = Serial.read();
              c2 = Serial.read();
              Off = (c1 << 8) | c2;
              offset = (float) Off;
              set_sinus_table_A(amp/100,offset/100);
          }
          else if (com==START_SYNCHRO) {
             start_synchro = 1;
            
          }
          else if (com==STOP_SYNCHRO) {
             synchro=0;
             start_synchro = 0;
             digitalWrite(7,LOW);
          }
       
    }
}  			     
    			     